@@ -1,6 +1,6 @@
 /* GNU/Linux native-dependent code for debugging multiple forks.
 
-   Copyright 2005, 2007, 2008 Free Software Foundation, Inc.
+   Copyright (C) 2005-2016 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -20,12 +20,8 @@
 struct fork_info;
 extern struct fork_info *add_fork (pid_t);
 extern struct fork_info *find_fork_pid (pid_t);
-extern void fork_save_infrun_state (struct fork_info *, int);
 extern void linux_fork_killall (void);
 extern void linux_fork_mourn_inferior (void);
-extern int  forks_exist_p (void);
-
-struct fork_info *fork_list;
-
-extern int detach_fork;
-
+extern void linux_fork_detach (const char *, int);
+extern int forks_exist_p (void);
+extern int linux_fork_checkpointing_p (int);
