@@ -1,6 +1,6 @@
 /* TUI Interpreter definitions for GDB, the GNU debugger.
 
-   Copyright (C) 2003-2013 Free Software Foundation, Inc.
+   Copyright (C) 2003-2016 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -29,12 +29,18 @@
 #include "tui/tui-win.h"
 #include "tui/tui.h"
 #include "tui/tui-io.h"
-#include "exceptions.h"
+#include "infrun.h"
+#include "observer.h"
+
+static struct ui_out *tui_ui_out (struct interp *self);
 
 /* Set to 1 when the TUI mode must be activated when we first start
    gdb.  */
 static int tui_start_enabled = 0;
 
+/* The TUI interpreter.  */
+static struct interp *tui_interp;
+
 /* Cleanup the tui before exiting.  */
 
 static void
@@ -45,16 +51,90 @@ tui_exit (void)
   tui_disable ();
 }
 
-/* True if TUI is the top-level interpreter.  */
-static int tui_is_toplevel = 0;
+/* Observers for several run control events.  If the interpreter is
+   quiet (i.e., another interpreter is being run with
+   interpreter-exec), print nothing.  */
+
+/* Observer for the normal_stop notification.  */
+
+static void
+tui_on_normal_stop (struct bpstats *bs, int print_frame)
+{
+  if (!interp_quiet_p (tui_interp))
+    {
+      if (print_frame)
+	print_stop_event (tui_ui_out (tui_interp));
+    }
+}
+
+/* Observer for the signal_received notification.  */
+
+static void
+tui_on_signal_received (enum gdb_signal siggnal)
+{
+  if (!interp_quiet_p (tui_interp))
+    print_signal_received_reason (tui_ui_out (tui_interp), siggnal);
+}
+
+/* Observer for the end_stepping_range notification.  */
+
+static void
+tui_on_end_stepping_range (void)
+{
+  if (!interp_quiet_p (tui_interp))
+    print_end_stepping_range_reason (tui_ui_out (tui_interp));
+}
+
+/* Observer for the signal_exited notification.  */
+
+static void
+tui_on_signal_exited (enum gdb_signal siggnal)
+{
+  if (!interp_quiet_p (tui_interp))
+    print_signal_exited_reason (tui_ui_out (tui_interp), siggnal);
+}
+
+/* Observer for the exited notification.  */
+
+static void
+tui_on_exited (int exitstatus)
+{
+  if (!interp_quiet_p (tui_interp))
+    print_exited_reason (tui_ui_out (tui_interp), exitstatus);
+}
+
+/* Observer for the no_history notification.  */
+
+static void
+tui_on_no_history (void)
+{
+  if (!interp_quiet_p (tui_interp))
+    print_no_history_reason (tui_ui_out (tui_interp));
+}
+
+/* Observer for the sync_execution_done notification.  */
+
+static void
+tui_on_sync_execution_done (void)
+{
+  if (!interp_quiet_p (tui_interp))
+    display_gdb_prompt (NULL);
+}
+
+/* Observer for the command_error notification.  */
+
+static void
+tui_on_command_error (void)
+{
+  if (!interp_quiet_p (tui_interp))
+    display_gdb_prompt (NULL);
+}
 
 /* These implement the TUI interpreter.  */
 
 static void *
 tui_init (struct interp *self, int top_level)
 {
-  tui_is_toplevel = top_level;
-
   /* Install exit handler to leave the screen in a good shape.  */
   atexit (tui_exit);
 
@@ -65,21 +145,19 @@ tui_init (struct interp *self, int top_level)
   if (ui_file_isatty (gdb_stdout))
     tui_initialize_readline ();
 
+  /* If changing this, remember to update cli-interp.c as well.  */
+  observer_attach_normal_stop (tui_on_normal_stop);
+  observer_attach_signal_received (tui_on_signal_received);
+  observer_attach_end_stepping_range (tui_on_end_stepping_range);
+  observer_attach_signal_exited (tui_on_signal_exited);
+  observer_attach_exited (tui_on_exited);
+  observer_attach_no_history (tui_on_no_history);
+  observer_attach_sync_execution_done (tui_on_sync_execution_done);
+  observer_attach_command_error (tui_on_command_error);
+
   return NULL;
 }
 
-/* True if enabling the TUI is allowed.  Example, if the top level
-   interpreter is MI, enabling curses will certainly lose.  */
-
-int
-tui_allowed_p (void)
-{
-  /* Only if TUI is the top level interpreter.  Also don't try to
-     setup curses (and print funny control characters) if we're not
-     outputting to a terminal.  */
-  return tui_is_toplevel && ui_file_isatty (gdb_stdout);
-}
-
 static int
 tui_resume (void *data)
 {
@@ -114,17 +192,6 @@ tui_suspend (void *data)
   return 1;
 }
 
-/* Display the prompt if we are silent.  */
-
-static int
-tui_display_prompt_p (void *data)
-{
-  if (interp_quiet_p (NULL))
-    return 0;
-  else
-    return 1;
-}
-
 static struct ui_out *
 tui_ui_out (struct interp *self)
 {
@@ -151,10 +218,10 @@ _initialize_tui_interp (void)
     tui_resume,
     tui_suspend,
     tui_exec,
-    tui_display_prompt_p,
     tui_ui_out,
+    NULL,
+    cli_command_loop
   };
-  struct interp *tui_interp;
 
   /* Create a default uiout builder for the TUI.  */
   tui_interp = interp_new (INTERP_TUI, &procs);
