@@ -1,21 +1,22 @@
 /* Print GOULD NPL instructions for GDB, the GNU debugger.
-   Copyright (C) 1986, 1987, 1989 Free Software Foundation, Inc.
+   Copyright (C) 1986-2016 Free Software Foundation, Inc.
 
-This file is part of GDB.
+   This file is part of GDB.
 
-GDB is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 1, or (at your option)
-any later version.
+   GDB is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 3, or (at your option)
+   any later version.
 
-GDB is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+   GDB is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
-You should have received a copy of the GNU General Public License
-along with GDB; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   You should have received a copy of the GNU General Public License
+   along with GDB; see the file COPYING3.  If not, write to
+   the Free Software Foundation, 51 Franklin Street - Fifth Floor,
+   Boston, MA 02110-1301, USA.  */
 
 struct gld_opcode
 {
@@ -36,7 +37,7 @@ struct gld_opcode
    of that opcode.
 
    The args component is a string containing characters
-   that are used to format the arguments to the instruction. */
+   that are used to format the arguments to the instruction.  */
 
 /* Kinds of operands:
    r  Register in first field
@@ -53,14 +54,12 @@ struct gld_opcode
    d  Offset field (14bits signed)
    S  Shift count field
 
-   any other characters are printed as is...
-*/
+   any other characters are printed as is...  */
 
 /* The assembler requires that this array be sorted as follows:
    all instances of the same mnemonic must be consecutive.
    All instances of the same mnemonic with the same number of operands
-   must be consecutive.
- */
+   must be consecutive.   */
 struct gld_opcode gld_opcodes[] =
 {
 { "lb",		0xb4080000,	0xfc080000,	"r,xOA,X",	4 },
